@@ -5,10 +5,10 @@ import random
 import warnings
 from abc import ABC, abstractmethod
 from itertools import groupby, zip_longest
-from typing import Any, Callable, Dict, Iterable, Iterator, List, Literal, Sequence, Set, Tuple, TypeVar, overload
+from typing import Any, Callable, Iterable, Iterator, Literal, Sequence, TypeVar, overload
 
 import vapoursynth as vs
-from vskernels import Catrom, get_matrix
+from vskernels import Catrom, get_matrix, get_prop
 from vsutil import depth, get_subsampling, get_w
 from vsutil import split as split_planes
 
@@ -16,11 +16,11 @@ from .dehardsub import hardsub_mask
 from .exceptions import ClipsAndNamedClipsError, InvalidFormatError, NotEqualFormatsError, VariableFormatError
 from .render import clip_async_render
 from .types import Direction
-from .util import check_variable, check_variable_format, check_variable_resolution, get_prop
+from .util import check_variable, check_variable_format, check_variable_resolution
 
 core = vs.core
 
-__all__: List[str] = [
+__all__ = [
     'compare', 'comp',
     'Comparer',
     'diff_hardsub_mask',
@@ -62,7 +62,7 @@ class Comparer(ABC):
     """
 
     def __init__(self,
-                 clips: Dict[str, vs.VideoNode] | Sequence[vs.VideoNode],
+                 clips: dict[str, vs.VideoNode] | Sequence[vs.VideoNode],
                  /,
                  *,
                  label_alignment: int = 7
@@ -74,23 +74,23 @@ class Comparer(ABC):
         if not isinstance(clips, (dict, Sequence)):
             raise TypeError(f"Comparer: Unexpected type {type(clips)} for `clips` argument!")
 
-        self.clips: List[vs.VideoNode] = list(clips.values()) if isinstance(clips, dict) else list(clips)
-        self.names: List[str] | None = list(clips.keys()) if isinstance(clips, dict) else None
+        self.clips = list(clips.values()) if isinstance(clips, dict) else list(clips)
+        self.names = list(clips.keys()) if isinstance(clips, dict) else None
 
         self.label_alignment = label_alignment
 
         self.num_clips = len(clips)
 
-        widths: Set[int] = {clip.width for clip in self.clips}
-        self.width: int | None = w if (w := widths.pop()) != 0 and len(widths) == 0 else None
+        widths = {clip.width for clip in self.clips}
+        self.width = w if (w := widths.pop()) != 0 and len(widths) == 0 else None
 
-        heights: Set[int] = {clip.height for clip in self.clips}
-        self.height: int | None = h if (h := heights.pop()) != 0 and len(heights) == 0 else None
+        heights = {clip.height for clip in self.clips}
+        self.height = h if (h := heights.pop()) != 0 and len(heights) == 0 else None
 
-        formats: Set[str | None] = {getattr(clip.format, 'name', None) for clip in self.clips}
-        self.format: str | None = formats.pop() if len(formats) == 1 else None
+        formats = {getattr(clip.format, 'name', None) for clip in self.clips}
+        self.format = formats.pop() if len(formats) == 1 else None
 
-    def _marked_clips(self) -> List[vs.VideoNode]:
+    def _marked_clips(self) -> list[vs.VideoNode]:
         """If a `name` is only space characters, `'   '`, for example, the name will not be overlaid on the clip."""
         if self.names:
             return [clip.text.Text(text=name, alignment=self.label_alignment) if name.strip() else clip
@@ -136,7 +136,7 @@ class Stack(Comparer):
     """
 
     def __init__(self,
-                 clips: Dict[str, vs.VideoNode] | Sequence[vs.VideoNode],
+                 clips: dict[str, vs.VideoNode] | Sequence[vs.VideoNode],
                  /,
                  *,
                  direction: Direction = Direction.HORIZONTAL,
@@ -178,7 +178,7 @@ class Interleave(Comparer):
     """
 
     def __init__(self,
-                 clips: Dict[str, vs.VideoNode] | Sequence[vs.VideoNode],
+                 clips: dict[str, vs.VideoNode] | Sequence[vs.VideoNode],
                  /,
                  *,
                  label_alignment: int = 7
@@ -242,10 +242,10 @@ class Tile(Comparer):
     """
 
     def __init__(self,
-                 clips: Dict[str, vs.VideoNode] | Sequence[vs.VideoNode],
+                 clips: dict[str, vs.VideoNode] | Sequence[vs.VideoNode],
                  /,
                  *,
-                 arrangement: List[List[int]] | None = None,
+                 arrangement: list[list[int]] | None = None,
                  label_alignment: int = 7
                  ) -> None:
         super().__init__(clips, label_alignment=label_alignment)
@@ -277,8 +277,8 @@ class Tile(Comparer):
                 for row in self.arrangement]
         return core.std.StackVertical(rows)
 
-    def _auto_arrangement(self) -> List[List[int]]:
-        def _grouper(iterable: Iterable[T], n: int, fillvalue: T | None = None) -> Iterator[Tuple[T, ...]]:
+    def _auto_arrangement(self) -> list[list[int]]:
+        def _grouper(iterable: Iterable[T], n: int, fillvalue: T | None = None) -> Iterator[tuple[T, ...]]:
             args = [iter(iterable)] * n
             return zip_longest(*args, fillvalue=fillvalue)
 
@@ -314,7 +314,7 @@ class Split(Stack):
     """
 
     def __init__(self,
-                 clips: Dict[str, vs.VideoNode] | Sequence[vs.VideoNode],
+                 clips: dict[str, vs.VideoNode] | Sequence[vs.VideoNode],
                  /,
                  *,
                  direction: Direction = Direction.HORIZONTAL,
@@ -365,7 +365,7 @@ class Split(Stack):
 
 
 def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
-            frames: List[int] | None = None,
+            frames: list[int] | None = None,
             rand_total: int | None = None,
             force_resample: bool = True, print_frame: bool = True,
             mismatch: bool = False) -> vs.VideoNode:
@@ -495,7 +495,7 @@ def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.Vide
     yuv_planes = split_planes(clip)
 
     if clip.format.color_family == vs.ColorFamily.YUV:
-        planes: Dict[str, vs.VideoNode] = {'Y': yuv_planes[0], 'U': yuv_planes[1], 'V': yuv_planes[2]}
+        planes: dict[str, vs.VideoNode] = {'Y': yuv_planes[0], 'U': yuv_planes[1], 'V': yuv_planes[2]}
     elif clip.format.color_family == vs.ColorFamily.RGB:
         planes = {'R': yuv_planes[0], 'G': yuv_planes[1], 'B': yuv_planes[2]}
     else:
@@ -538,9 +538,9 @@ def diff(*clips: vs.VideoNode,
          height: int = ...,
          interleave: bool = ...,
          return_ranges: Literal[True] = True,
-         exclusion_ranges: Sequence[int | Tuple[int, int]] | None = ...,
+         exclusion_ranges: Sequence[int | tuple[int, int]] | None = ...,
          diff_func: Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode] = ...,
-         **namedclips: vs.VideoNode) -> Tuple[vs.VideoNode, List[Tuple[int, int]]]:
+         **namedclips: vs.VideoNode) -> tuple[vs.VideoNode, list[tuple[int, int]]]:
     ...
 
 
@@ -550,7 +550,7 @@ def diff(*clips: vs.VideoNode,
          height: int = ...,
          interleave: bool = ...,
          return_ranges: Literal[False],
-         exclusion_ranges: Sequence[int | Tuple[int, int]] | None = ...,
+         exclusion_ranges: Sequence[int | tuple[int, int]] | None = ...,
          diff_func: Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode] = ...,
          **namedclips: vs.VideoNode) -> vs.VideoNode:
     ...
@@ -561,9 +561,9 @@ def diff(*clips: vs.VideoNode,
          height: int = 288,
          interleave: bool = False,
          return_ranges: bool = False,
-         exclusion_ranges: Sequence[int | Tuple[int, int]] | None = None,
+         exclusion_ranges: Sequence[int | tuple[int, int]] | None = None,
          diff_func: Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode] = lambda a, b: core.std.MakeDiff(a, b),
-         **namedclips: vs.VideoNode) -> vs.VideoNode | Tuple[vs.VideoNode, List[Tuple[int, int]]]:
+         **namedclips: vs.VideoNode) -> vs.VideoNode | tuple[vs.VideoNode, list[tuple[int, int]]]:
     """
     Create a standard :py:class:`lvsfunc.comparison.Stack` between frames from two clips that have differences.
 
@@ -617,12 +617,12 @@ def diff(*clips: vs.VideoNode,
     if thr >= 128:
         raise ValueError("diff: `thr` must be below 128!")
 
-    if clips and not all([c.format for c in clips]):  # noqa
+    if clips and not all([c.format for c in clips]):
         raise VariableFormatError("diff")
     elif namedclips and not all([nc.format for nc in namedclips.values()]):
         raise VariableFormatError("diff")
 
-    def _to_ranges(iterable: List[int]) -> Iterable[Tuple[int, int]]:
+    def _to_ranges(iterable: list[int]) -> Iterable[tuple[int, int]]:
         iterable = sorted(set(iterable))
         for _, group in groupby(enumerate(iterable), lambda t: t[1] - t[0]):
             groupl = list(group)
@@ -660,7 +660,7 @@ def diff(*clips: vs.VideoNode,
     frames.sort()
 
     if exclusion_ranges:
-        r: List[int] = []
+        r: list[int] = []
         for e in exclusion_ranges:
             if isinstance(e, int):
                 e = (e, e)
