@@ -7,10 +7,11 @@ import warnings
 from fractions import Fraction
 from functools import partial
 from pathlib import Path
-from typing import Any, Dict, List, Sequence
+from typing import Any, Sequence, cast
 
 import vapoursynth as vs
-from vskernels import Bicubic, BicubicDidee, Catrom, Kernel, get_kernel
+from vsexprtools import mod2, mod4
+from vskernels import Bicubic, BicubicDidee, Catrom, Kernel, get_kernel, get_prop
 from vsrgtools import repair
 from vsutil import Dither, depth, get_depth, get_neutral_value, get_w, get_y, scale_value
 
@@ -18,11 +19,11 @@ from .comparison import Stack
 from .exceptions import InvalidFramerateError, TopFieldFirstError
 from .render import clip_async_render, get_render_progress
 from .types import Direction
-from .util import check_variable, check_variable_format, force_mod, get_prop
+from .util import check_variable, check_variable_format
 
 core = vs.core
 
-__all__: List[str] = [
+__all__ = [
     'check_patterns',
     'deblend',
     'decomb',
@@ -106,7 +107,7 @@ def seek_cycle(clip: vs.VideoNode, write_props: bool = True, scale: int = -1) ->
     clip = Catrom().scale(clip, width, height)
 
     # Downscaling for the cycle clips
-    clip_down = BicubicDidee().scale(clip, force_mod(width/4, 2), force_mod(height/4, 2))
+    clip_down = BicubicDidee().scale(clip, mod2(width/4), mod2(height/4))
     if write_props:
         clip_down = core.std.FrameEval(clip_down, partial(check_combed, clip=clip_down), clip_down).text.FrameNum(2)
     blank_frame = clip_down.std.BlankClip(length=1, color=[0] * 3)
@@ -117,15 +118,15 @@ def seek_cycle(clip: vs.VideoNode, write_props: bool = True, scale: int = -1) ->
 
     # Cycling
     cycle_clips = [pad_a, pad_b, pad_c, pad_d, pad_e]
-    pad_x = [pad_a.std.BlankClip(force_mod(pad_a.width/15))] * 4
+    pad_x = [pad_a.std.BlankClip(mod4(pad_a.width/15))] * 4
     cycle = cycle_clips + pad_x  # no shot this can't be done way cleaner
     cycle[::2], cycle[1::2] = cycle_clips, pad_x
 
     # Final stacking
     stack_abcde = Stack(cycle).clip
 
-    vert_pad = stack_abcde.std.BlankClip(height=force_mod(stack_abcde.height / 5, 2))
-    horz_pad = clip.std.BlankClip(force_mod((stack_abcde.width-clip.width) / 2, 2))
+    vert_pad = stack_abcde.std.BlankClip(height=mod2(stack_abcde.height / 5))
+    horz_pad = clip.std.BlankClip(mod2((stack_abcde.width-clip.width) / 2))
 
     stack = Stack([horz_pad, clip, horz_pad]).clip
     return Stack([vert_pad, stack, vert_pad, stack_abcde], direction=Direction.VERTICAL).clip
@@ -185,8 +186,8 @@ def tivtc_vfr(clip: vs.VideoNode,
               tdec_in: Path | str = f".ivtc/{main_file}metrics.txt",
               timecodes_out: Path | str = f".ivtc/{main_file}timecodes.txt",
               decimate: int | bool = True,
-              tfm_args: Dict[str, Any] = {},
-              tdecimate_args: Dict[str, Any] = {}) -> vs.VideoNode:
+              tfm_args: dict[str, Any] = {},
+              tdecimate_args: dict[str, Any] = {}) -> vs.VideoNode:
     """
     Perform TFM and TDecimate on a clip that is supposed to be VFR.
 
@@ -245,8 +246,8 @@ def tivtc_vfr(clip: vs.VideoNode,
                       "If it still doesn't work, open the ``.ivtc`` directory and check if the files are 0kb. "
                       "If they are, delete them and run the function again.'")
 
-        tfm_analysis: Dict[str, Any] = {**tfm_args, 'output': str(tfm_f)}
-        tdec_analysis: Dict[str, Any] = {'mode': 4, **tdecimate_args, 'output': str(tdec_f)}
+        tfm_analysis: dict[str, Any] = {**tfm_args, 'output': str(tfm_f)}
+        tdec_analysis: dict[str, Any] = {'mode': 4, **tdecimate_args, 'output': str(tdec_f)}
 
         ivtc_clip = core.tivtc.TFM(clip, **tfm_analysis)
         ivtc_clip = core.tivtc.TDecimate(ivtc_clip, **tdec_analysis)
@@ -335,10 +336,10 @@ def decomb(clip: vs.VideoNode,
            decimate: bool = True, vinv: bool = False,
            rep: int | Sequence[int] = 0,
            show_mask: bool = False,
-           tfm_args: Dict[str, Any] = {},
-           tdec_args: Dict[str, Any] = {},
-           vinv_args: Dict[str, Any] = {},
-           qtgmc_args: Dict[str, Any] = {}) -> vs.VideoNode:
+           tfm_args: dict[str, Any] = {},
+           tdec_args: dict[str, Any] = {},
+           vinv_args: dict[str, Any] = {},
+           qtgmc_args: dict[str, Any] = {}) -> vs.VideoNode:
     """
     Perform relatively aggressive filtering to get rid of the combing on a interlaced/telecined source.
 
@@ -382,10 +383,10 @@ def decomb(clip: vs.VideoNode,
 
     VFM_TFF = int(tff)
 
-    tfm_kwargs: Dict[str, Any] = dict(order=VFM_TFF, mode=mode, chroma=True)
+    tfm_kwargs: dict[str, Any] = dict(order=VFM_TFF, mode=mode, chroma=True)
     tfm_kwargs |= tfm_args  # chroma set to True by default to match VFM
 
-    qtgmc_kwargs: Dict[str, Any] = dict(SourceMatch=3, Lossless=2, TR0=1, TR1=2, TR2=3, FPSDivisor=2, TFF=tff)
+    qtgmc_kwargs: dict[str, Any] = dict(SourceMatch=3, Lossless=2, TR0=1, TR1=2, TR2=3, FPSDivisor=2, TFF=tff)
     qtgmc_kwargs |= qtgmc_args
 
     def _pp(n: int, f: vs.VideoFrame, clip: vs.VideoNode, pp: vs.VideoNode) -> vs.VideoNode:
@@ -483,7 +484,7 @@ def fix_telecined_fades(clip: vs.VideoNode, tff: bool | int | None = None,
 
     :raises TopFieldFirstError:     No automatic ``tff`` can be determined.
     """
-    def _ftf(n: int, f: List[vs.VideoFrame]) -> vs.VideoNode:
+    def _ftf(n: int, f: list[vs.VideoFrame]) -> vs.VideoNode:
         avg = (get_prop(f[0], 'PlaneStatsAverage', float),
                get_prop(f[1], 'PlaneStatsAverage', float))
 
@@ -492,7 +493,7 @@ def fix_telecined_fades(clip: vs.VideoNode, tff: bool | int | None = None,
             fixed = (sep[0].akarin.Expr(f"x {mean} {avg[0]} / dup {thr} <= swap 1 ? *"),
                      sep[1].akarin.Expr(f"x {mean} {avg[1]} / *"))
         else:
-            fixed = sep  # type: ignore
+            fixed = cast(tuple[vs.VideoNode, vs.VideoNode], sep)
 
         return core.std.Interleave(fixed).std.DoubleWeave()[::2]
 
@@ -521,7 +522,7 @@ def fix_telecined_fades(clip: vs.VideoNode, tff: bool | int | None = None,
 
 def pulldown_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
                      interlaced: bool = True, dec: bool | None = None,
-                     bob_clip: vs.VideoNode | None = None, qtgmc_args: Dict[str, Any] = {}
+                     bob_clip: vs.VideoNode | None = None, qtgmc_args: dict[str, Any] = {}
                      ) -> vs.VideoNode:
     """
     Deinterlacing function for interlaced credits (60i/30p) on top of telecined video (24p).
@@ -558,10 +559,16 @@ def pulldown_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None
     :raises InvalidFramerateError:  Bobbed clip does not have a framerate of 60000/1001 (59.94)
     """
     try:
-        from havsfunc import QTGMC, DitherLumaRebuild
+        from havsfunc import QTGMC
     except ModuleNotFoundError:
         raise ModuleNotFoundError("pulldown_credits: missing dependency `havsfunc`!")
 
+    try:
+        from vsdenoise import prefilter_to_full_range
+    except ModuleNotFoundError:
+        from havsfunc import DitherLumaRebuild as prefilter_to_full_range  # type: ignore
+        warnings.warn("pulldown_credits: missing dependency `vsdenoise`!", ImportWarning)
+
     assert check_variable(clip, "pulldown_credits")
 
     if clip.fps != Fraction(30000, 1001):
@@ -572,7 +579,7 @@ def pulldown_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None
     elif isinstance(tff, (bool, int)):
         clip = clip.std.SetFieldBased(int(tff) + 1)
 
-    qtgmc_kwargs: Dict[str, Any] = dict(SourceMatch=3, Lossless=2, TR0=2, TR1=2, TR2=3, Preset="Placebo")
+    qtgmc_kwargs: dict[str, Any] = dict(SourceMatch=3, Lossless=2, TR0=2, TR1=2, TR2=3, Preset="Placebo")
     qtgmc_kwargs |= qtgmc_args
     qtgmc_kwargs |= dict(FPSDivisor=1, TFF=tff or bool(get_prop(clip.get_frame(0), '_FieldBased', int) - 1))
 
@@ -632,7 +639,7 @@ def pulldown_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None
             else:
                 jitter = bobbed.std.SelectEvery(5, [3 - invpos, 4 - invpos])
 
-        jsup_pre = DitherLumaRebuild(jitter, s0=1).mv.Super(pel=2)
+        jsup_pre = prefilter_to_full_range(jitter, 1.0).mv.Super(pel=2)
         jsup = jitter.mv.Super(pel=2, levels=1)
         vect_f = jsup_pre.mv.Analyse(blksize=blksize, isb=False, delta=1, overlap=overlap)
         vect_b = jsup_pre.mv.Analyse(blksize=blksize, isb=True, delta=1, overlap=overlap)
@@ -674,13 +681,13 @@ def pulldown_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None
             else:
                 c2 = bobbed.std.SelectEvery(10, [offset, 1 + offset, 6 + offset, 5 + offset])
 
-        super1_pre = DitherLumaRebuild(c1, s0=1).mv.Super(pel=2)
+        super1_pre = prefilter_to_full_range(c1, 1.0).mv.Super(pel=2)
         super1 = c1.mv.Super(pel=2, levels=1)
         vect_f1 = super1_pre.mv.Analyse(blksize=blksize, isb=False, delta=1, overlap=overlap)
         vect_b1 = super1_pre.mv.Analyse(blksize=blksize, isb=True, delta=1, overlap=overlap)
         fix1 = c1.mv.FlowInter(super1, vect_b1, vect_f1, time=50 + direction * 25).std.SelectEvery(4, [0, 2])
 
-        super2_pre = DitherLumaRebuild(c2, s0=1).mv.Super(pel=2)
+        super2_pre = prefilter_to_full_range(c2, 1.0).mv.Super(pel=2)
         super2 = c2.mv.Super(pel=2, levels=1)
         vect_f2 = super2_pre.mv.Analyse(blksize=blksize, isb=False, delta=1, overlap=overlap)
         vect_b2 = super2_pre.mv.Analyse(blksize=blksize, isb=True, delta=1, overlap=overlap)
@@ -737,7 +744,7 @@ def _check_pattern(clip: vs.VideoNode, pattern: int = 0) -> bool:
     clip = sivtc(clip, pattern)
     clip = core.tdm.IsCombed(clip)
 
-    frames: List[int] = []
+    frames: list[int] = []
 
     def _cb(n: int, f: vs.VideoFrame) -> None:
         if get_prop(f, '_Combed', int):
