@@ -3,22 +3,21 @@ from __future__ import annotations
 import math
 from abc import ABC, abstractmethod
 from functools import partial
-from typing import Any, Callable, Dict, List, Sequence, Tuple
+from typing import Any, Callable, Sequence
 
 import vapoursynth as vs
+from vsexprtools.util import normalise_planes
 from vsrgtools import removegrain
-from vsrgtools.util import normalise_planes
 from vsutil import Range as CRange
 from vsutil import depth, get_depth, get_y, iterate, join, split
 
-from . import util
 from .types import Position, Range, Shapes, Size
-from .util import check_variable, check_variable_resolution, replace_ranges, scale_peak, scale_thresh
+from .util import check_variable, check_variable_resolution, quick_resample, replace_ranges, scale_peak, scale_thresh
 
 core = vs.core
 
 
-__all__: List[str] = [
+__all__ = [
     'BoundingBox',
     'DeferredMask',
     'detail_mask_neo',
@@ -60,7 +59,7 @@ def detail_mask(clip: vs.VideoNode, sigma: float | None = None,
     brz_a = scale_thresh(brz_a, clip)
     brz_b = scale_thresh(brz_b, clip)
 
-    blur = (util.quick_resample(clip, partial(core.bilateral.Gaussian, sigma=sigma))
+    blur = (quick_resample(clip, partial(core.bilateral.Gaussian, sigma=sigma))
             if sigma else clip)
 
     mask_a = range_mask(get_y(blur), rad=rad)
@@ -343,7 +342,7 @@ class BoundingBox():
     pos: Position
     size: Size
 
-    def __init__(self, pos: Position | Tuple[int, int], size: Size | Tuple[int, int]):
+    def __init__(self, pos: Position | tuple[int, int], size: Size | tuple[int, int]):
         self.pos = pos if isinstance(pos, Position) else Position(pos[0], pos[1])
         self.size = size if isinstance(size, Size) else Size(size[0], size[1])
 
@@ -364,7 +363,7 @@ class BoundingBox():
 
         mask_fmt: vs.VideoFormat = ref.format.replace(color_family=vs.GRAY, subsampling_w=0, subsampling_h=0)
         white: float = 1 if mask_fmt.sample_type == vs.FLOAT else (1 << ref.format.bits_per_sample) - 1
-        mask: vs.VideoNode = core.std.BlankClip(ref, color=white, length=1, format=mask_fmt.id)
+        mask: vs.VideoNode = ref.std.BlankClip(color=white, format=mask_fmt.id, keep=True)
         mask = mask.std.Crop(self.pos.x, ref.width - (self.pos.x + self.size.x),
                              self.pos.y, ref.height - (self.pos.y + self.size.y))
         mask = mask.std.AddBorders(self.pos.x, ref.width - (self.pos.x + self.size.x),
@@ -391,15 +390,15 @@ class DeferredMask(ABC):
     :raises ValueError: Reference frame and ranges mismatch.
     """
 
-    ranges: List[Range]
+    ranges: list[Range]
     bound: BoundingBox | None
-    refframes: List[int | None]
+    refframes: list[int | None]
     blur: bool
 
-    def __init__(self, ranges: Range | List[Range] | None = None,
-                 bound: BoundingBox | Tuple[Tuple[int, int], Tuple[int, int]] | None = None,
+    def __init__(self, ranges: Range | list[Range] | None = None,
+                 bound: BoundingBox | tuple[tuple[int, int], tuple[int, int]] | None = None,
                  *,
-                 blur: bool = False, refframes: int | List[int] | None = None):
+                 blur: bool = False, refframes: int | list[int] | None = None):
         self.ranges = ranges if isinstance(ranges, list) else [(0, None)] if ranges is None else [ranges]
         self.blur = blur
 
@@ -465,8 +464,8 @@ def mt_xxpand_multi(clip: vs.VideoNode,
                     mode: Shapes | int = Shapes.ELLIPSE,
                     start: int = 0,
                     m__imum: Callable[..., vs.VideoNode] = core.std.Maximum,
-                    planes: List[int] = [0, 1, 2],
-                    **m_params: Any) -> List[vs.VideoNode]:
+                    planes: list[int] = [0, 1, 2],
+                    **m_params: Any) -> list[vs.VideoNode]:
     """
     Mask expanding/inpanding function written by `Zastin <https://github.com/kgrabs>`_.
 
@@ -476,7 +475,7 @@ def mt_xxpand_multi(clip: vs.VideoNode,
 
     planes = normalise_planes(clip, planes)
 
-    params: Dict[str, Any] = dict(planes=planes)
+    params: dict[str, Any] = dict(planes=planes)
     params |= m_params
 
     sh = sh or sw
@@ -487,7 +486,7 @@ def mt_xxpand_multi(clip: vs.VideoNode,
         case Shapes.LOSANGE: coordinates = [[0, 1, 0, 1, 1, 0, 1, 0]] * 3
         case _: coordinates = [[1] * 8] * 3
 
-    clips: List[vs.VideoNode] = [clip]
+    clips: list[vs.VideoNode] = [clip]
     end = int(min(sw, sh)) + start
 
     for x in range(start, end):
