@@ -1,29 +1,34 @@
 from __future__ import annotations
 
-from typing import Any, Dict, List
+import warnings
+from typing import Any
 
 import vapoursynth as vs
-from vskernels import Bicubic, Kernel, Matrix, get_kernel
-from vsutil import Dither, Range, depth, get_depth, get_y, iterate, join, plane
+from vskernels import Bicubic, Kernel, Matrix, get_kernel, get_prop
+from vsutil import Dither, depth, get_depth, get_y, join, plane
 
-from .util import check_variable, get_prop
+from .util import check_variable
 
 core = vs.core
 
 
-__all__: List[str] = [
-    'bm3d',
-    'chickendream'
+__all__ = [
+    'chickendream',
+    # Deprecated
+    'bm3d'
 ]
 
 
-def bm3d(clip: vs.VideoNode, sigma: float | List[float] = 0.75,
-         radius: int | List[int] | None = None, ref: vs.VideoNode | None = None,
+def bm3d(clip: vs.VideoNode, sigma: float | list[float] = 0.75,
+         radius: int | list[int] | None = None, ref: vs.VideoNode | None = None,
          pre: vs.VideoNode | None = None, refine: int = 1, matrix_s: str = "709",
-         basic_args: Dict[str, Any] = {}, final_args: Dict[str, Any] = {}) -> vs.VideoNode:
+         basic_args: dict[str, Any] = {}, final_args: dict[str, Any] = {}) -> vs.VideoNode:
     """
     BM3D denoising filter using the CPU.
 
+    .. warning::
+        | This function has been deprecated! It will be removed in a future commit.
+
     Dependencies:
 
     * `VapourSynth-BM3D <https://github.com/HomeOfVapourSynthEvolution/VapourSynth-BM3D>`_
@@ -46,60 +51,18 @@ def bm3d(clip: vs.VideoNode, sigma: float | List[float] = 0.75,
     :raises ValueError:     Invalid number of sigma parameters were passed.
     :raises ValueError:     Invalid number of radii parameters were passed.
     """
-    assert check_variable(clip, "bm3d")
+    try:
+        import vsdenoise
+    except ModuleNotFoundError:
+        raise ModuleNotFoundError("bm3d: missing dependency `vsdenoise`!")
 
-    is_gray = clip.format.color_family == vs.GRAY
+    warnings.warn('lvsfunc.bm3d: deprecated in favor of vsdenoise.BM3D!', DeprecationWarning)
 
-    def to_opp(clip: vs.VideoNode) -> vs.VideoNode:
-        return clip.resize.Bicubic(format=vs.RGBS, matrix_in_s=matrix_s).bm3d.RGB2OPP(sample=1)
+    vsd_bm3d = vsdenoise.BM3D(clip, sigma, radius, vsdenoise.Profile.NORMAL, pre, ref, refine)
+    vsd_bm3d.basic_args |= basic_args
+    vsd_bm3d.final_args |= final_args
 
-    def to_fullgray(clip: vs.VideoNode) -> vs.VideoNode:
-        return get_y(clip).resize.Point(format=vs.GRAYS, range_in=Range.LIMITED, range=Range.FULL)
-
-    sigmal = [sigma] * 3 if not isinstance(sigma, list) else sigma + [sigma[-1]]*(3-len(sigma))
-    sigmal = [sigmal[0], 0, 0] if is_gray else sigmal
-    is_gray = True if sigmal[1] == 0 and sigmal[2] == 0 else is_gray
-
-    if len(sigmal) != 3:
-        raise ValueError("bm3d: 'invalid number of sigma parameters supplied!'")
-
-    radiusl = [0, 0] if radius is None else [radius] * 2 if not isinstance(radius, list) \
-        else radius + [radius[-1]]*(2-len(radius))
-
-    if len(radiusl) != 2:
-        raise ValueError("bm3d: 'invalid number or radius parameters supplied!'")
-
-    if sigmal[0] == 0 and sigmal[1] == 0 and sigmal[2] == 0:
-        return clip
-
-    pre = pre if pre is None else to_opp(pre) if not is_gray else to_fullgray(pre)
-
-    def basic(clip: vs.VideoNode) -> vs.VideoNode:
-        return clip.bm3d.Basic(sigma=sigmal, ref=pre, matrix=100, **basic_args) if radiusl[0] < 1 \
-            else clip.bm3d.VBasic(sigma=sigmal, ref=pre, radius=radiusl[0], matrix=100, **basic_args) \
-            .bm3d.VAggregate(radius=radiusl[0], sample=1)
-
-    clip_in = to_opp(clip) if not is_gray else to_fullgray(clip)
-    refv = basic(clip_in) if ref is None else to_opp(ref) if not is_gray else to_fullgray(ref)
-
-    def final(clip: vs.VideoNode) -> vs.VideoNode:
-        return clip.bm3d.Final(sigma=sigmal, ref=refv, matrix=100, **final_args) if radiusl[1] < 1 \
-            else clip.bm3d.VFinal(sigma=sigmal, ref=refv, radius=radiusl[1], matrix=100, **final_args) \
-            .bm3d.VAggregate(radius=radiusl[1], sample=1)
-
-    den = iterate(clip_in, final, refine)
-
-    # boil everything back down to whatever input we had
-    den = den.bm3d.OPP2RGB(sample=1).resize.Bicubic(format=clip.format.id, matrix_s=matrix_s) if not is_gray \
-        else den.resize.Point(format=clip.format.replace(color_family=vs.GRAY, subsampling_w=0, subsampling_h=0).id,
-                              range_in=Range.FULL, range=Range.LIMITED)
-    # merge source chroma if it exists and we didn't denoise it
-    den = core.std.ShufflePlanes([den, clip], planes=[0, 1, 2], colorfamily=vs.YUV) \
-        if is_gray and clip.format.color_family == vs.YUV else den
-    # sub clip luma back in if we only denoised chroma
-    den = den if sigmal[0] != 0 else core.std.ShufflePlanes([clip, den], planes=[0, 1, 2], colorfamily=vs.YUV)
-
-    return den
+    return vsd_bm3d.clip
 
 
 def chickendream(clip: vs.VideoNode, sigma: float = 0.35,
@@ -167,7 +130,7 @@ def chickendream(clip: vs.VideoNode, sigma: float = 0.35,
     if matrix is None:
         matrix = get_prop(clip.get_frame(0), "_Matrix", int)
 
-    targ_matrix = vs.MatrixCoefficients(matrix)
+    targ_matrix = Matrix(matrix)
 
     if is_rgb:
         input_clip = clip_32
