@@ -4,36 +4,32 @@ import colorsys
 import random
 import warnings
 from functools import partial, wraps
-from typing import Any, Callable, Dict, List, Tuple, Type, cast
+from typing import Any, Callable, cast
 
 import vapoursynth as vs
 from typing_extensions import TypeGuard
-from vskernels import Bicubic, Kernel, Matrix, get_kernel, get_matrix
+from vskernels import Bicubic, Kernel, VNodeCallable, get_kernel, get_matrix, get_prop
 from vsutil import depth, get_subsampling, get_w, get_y
 
-from .exceptions import InvalidFormatError, InvalidMatrixError, VariableFormatError, VariableResolutionError
-from .types import CURVES, Coefs, F, Range, T, _VideoNode
+from .exceptions import InvalidFormatError, VariableFormatError, VariableResolutionError
+from .types import Range, _VideoNode
 
 core = vs.core
 
 
-__all__: List[str] = [
+__all__ = [
     'allow_variable',
     'check_variable',
     'chroma_injector',
     'colored_clips',
-    'force_mod',
     'frames_since_bookmark',
-    'get_coefs',
-    'get_matrix_curve',
-    'get_prop',
     'load_bookmarks',
     'normalize_ranges',
     'padder',
     'quick_resample',
     'replace_ranges', 'rfs',
     'scale_peak',
-    'scale_thresh',
+    'scale_thresh'
 ]
 
 
@@ -72,31 +68,7 @@ def quick_resample(clip: vs.VideoNode,
     return depth(filtered, clip.format.bits_per_sample)
 
 
-def get_prop(frame: vs.VideoFrame, key: str, t: Type[T]) -> T:
-    """
-    Get FrameProp ``prop`` from frame ``frame`` with expected type ``t`` to satisfy the type checker.
-
-    :param frame:           Frame containing props.
-    :param key:             Prop to get.
-    :param t:               Type of prop.
-
-    :return:                frame.prop[key].
-
-    :raises KeyError:       ``key`` is not found in props.
-    :raises ValueError:     Returns a prop of the wrong type.
-    """
-    try:
-        prop = frame.props[key]
-    except KeyError:
-        raise KeyError(f"get_prop: 'Key {key} not present in props!'")
-
-    if not isinstance(prop, t):
-        raise ValueError(f"get_prop: 'Key {key} did not contain expected type: Expected {t} got {type(prop)}!'")
-
-    return prop
-
-
-def normalize_ranges(clip: vs.VideoNode, ranges: Range | List[Range]) -> List[Tuple[int, int]]:
+def normalize_ranges(clip: vs.VideoNode, ranges: Range | list[Range]) -> list[tuple[int, int]]:
     r"""
     Normalize :py:func:`lvsfunc.types.Range`\(s) to a list of inclusive positive integer ranges.
 
@@ -133,7 +105,7 @@ def normalize_ranges(clip: vs.VideoNode, ranges: Range | List[Range]) -> List[Tu
 
 def replace_ranges(clip_a: vs.VideoNode,
                    clip_b: vs.VideoNode,
-                   ranges: Range | List[Range] | None,
+                   ranges: Range | list[Range] | None,
                    exclusive: bool = False,
                    use_plugin: bool = True) -> vs.VideoNode:
     """
@@ -246,15 +218,6 @@ def scale_peak(value: float, peak: float) -> float:
     return value * peak / 255
 
 
-def force_mod(x: float, mod: int = 4) -> int:
-    """
-    Force output to fit a specific MOD.
-
-    Minimum returned value will always be mod².
-    """
-    return mod ** 2 if x < mod ** 2 else int(x / mod + 0.5) * mod
-
-
 def padder(clip: vs.VideoNode,
            left: int = 32, right: int = 32,
            top: int = 32, bottom: int = 32) -> vs.VideoNode:
@@ -291,25 +254,6 @@ def padder(clip: vs.VideoNode,
     return core.fb.FillBorders(scaled, left=left, right=right, top=top, bottom=bottom)
 
 
-def get_coefs(curve: vs.TransferCharacteristics) -> Coefs:
-    """Return transfer coefs."""
-    srgb = Coefs(0.04045, 12.92, 0.055, 2.4)
-    bt709 = Coefs(0.08145, 4.5, 0.0993, 2.22222)
-    smpte240m = Coefs(0.0912, 4.0, 0.1115, 2.22222)
-    bt2020 = Coefs(0.08145, 4.5, 0.0993, 2.22222)
-
-    gamma_linear_map = {
-        vs.TransferCharacteristics.TRANSFER_IEC_61966_2_1: srgb,
-        vs.TransferCharacteristics.TRANSFER_BT709: bt709,
-        vs.TransferCharacteristics.TRANSFER_BT601: bt709,
-        vs.TransferCharacteristics.TRANSFER_ST240_M: smpte240m,
-        vs.TransferCharacteristics.TRANSFER_BT2020_10: bt2020,
-        vs.TransferCharacteristics.TRANSFER_BT2020_12: bt2020
-    }
-
-    return gamma_linear_map[curve]
-
-
 def check_variable_format(clip: vs.VideoNode, function: str) -> TypeGuard[_VideoNode]:
     """
     Check for variable format and return an error if found.
@@ -343,23 +287,7 @@ def check_variable(clip: vs.VideoNode, function: str) -> TypeGuard[_VideoNode]:
     return True
 
 
-def get_matrix_curve(matrix: Matrix) -> CURVES:
-    """
-    Return the matrix curve based on a given ``matrix``.
-
-    :raises InvalidMatrixError:     An invalid ``matrix`` is passed.
-    """
-    match matrix:
-        case Matrix.BT709: return vs.TransferCharacteristics.TRANSFER_BT709
-        case Matrix.BT470BG | Matrix.SMPTE170M: return vs.TransferCharacteristics.TRANSFER_BT601
-        case Matrix.SMPTE240M: return vs.TransferCharacteristics.TRANSFER_ST240_M
-        case Matrix.CHROMA_DERIVED_C: return vs.TransferCharacteristics.TRANSFER_IEC_61966_2_1
-        case Matrix.ICTCP: return vs.TransferCharacteristics.TRANSFER_BT2020_10
-        # case 15: return vs.TransferCharacteristics.TRANSFER_BT2020_12
-        case _: raise InvalidMatrixError("get_matrix_curve", message="{func}: 'An invalid matrix value was passed!'")
-
-
-def load_bookmarks(bookmark_path: str) -> List[int]:
+def load_bookmarks(bookmark_path: str) -> list[int]:
     """
     Load VSEdit bookmarks.
 
@@ -379,7 +307,7 @@ def load_bookmarks(bookmark_path: str) -> List[int]:
     return bookmarks
 
 
-def frames_since_bookmark(clip: vs.VideoNode, bookmarks: List[int]) -> vs.VideoNode:
+def frames_since_bookmark(clip: vs.VideoNode, bookmarks: list[int]) -> vs.VideoNode:
     """
     Display frames since last bookmark to create easily reusable scenefiltering.
 
@@ -390,7 +318,7 @@ def frames_since_bookmark(clip: vs.VideoNode, bookmarks: List[int]) -> vs.VideoN
 
     :return:            Clip with bookmarked frames.
     """
-    def _frames_since_bookmark(n: int, clip: vs.VideoNode, bookmarks: List[int]) -> vs.VideoNode:
+    def _frames_since_bookmark(n: int, clip: vs.VideoNode, bookmarks: list[int]) -> vs.VideoNode:
         for i, bookmark in enumerate(bookmarks):
             frames_since = n - bookmark
 
@@ -404,7 +332,7 @@ def frames_since_bookmark(clip: vs.VideoNode, bookmarks: List[int]) -> vs.VideoN
     return core.std.FrameEval(clip, partial(_frames_since_bookmark, clip=clip, bookmarks=bookmarks))
 
 
-def chroma_injector(func: F) -> F:
+def chroma_injector(func: VNodeCallable) -> VNodeCallable:
     """
     Inject reference chroma.
 
@@ -489,7 +417,7 @@ def chroma_injector(func: F) -> F:
         else:
             return allow_variable()(get_y)(result)
 
-    return cast(F, inner)
+    return cast(VNodeCallable, inner)
 
 
 def colored_clips(amount: int,
@@ -497,7 +425,7 @@ def colored_clips(amount: int,
                   rand: bool = True,
                   seed: bytearray | bytes | float | str | None = None,
                   **kwargs: Any
-                  ) -> List[vs.VideoNode]:
+                  ) -> list[vs.VideoNode]:
     r"""
     Return a list of BlankClips with unique colors in sequential or random order.
 
@@ -530,12 +458,11 @@ def colored_clips(amount: int,
     if not (0 < max_hue <= 360):
         raise ValueError("colored_clips: `max_hue` must be greater than 0 and less than 360 degrees!")
 
-    blank_clip_args: Dict[str, Any] = {'keep': 1, **kwargs}
+    blank_clip_args: dict[str, Any] = {'keep': 1, **kwargs}
 
-    hues: List[float] = [i * max_hue / (amount - 1) for i in range(amount - 1)]
-    hues.append(max_hue)
+    hues = [i * max_hue / (amount - 1) for i in range(amount - 1)] + [max_hue]
 
-    hls_color_list: List[Tuple[float, float, float]] = [colorsys.hls_to_rgb(h / 360, 0.5, 1) for h in hues]
+    hls_color_list = [colorsys.hls_to_rgb(h / 360, 0.5, 1) for h in hues]
     rgb_color_list = [[int(f * 255) for f in color] for color in hls_color_list]
 
     if rand:
@@ -577,7 +504,7 @@ def allow_variable(width: int | None = None, height: int | None = None,
     if height is not None:
         width = width if width else get_w(height)
 
-    def inner(func: F) -> F:
+    def inner(func: VNodeCallable) -> VNodeCallable:
         @wraps(func)
         def inner2(clip: vs.VideoNode, *args: Any, **kwargs: Any) -> vs.VideoNode:
             def frameeval_wrapper(n: int, f: vs.VideoFrame) -> vs.VideoNode:
@@ -588,7 +515,7 @@ def allow_variable(width: int | None = None, height: int | None = None,
             clip_out = clip_out.resize.Point(width, height) if width and height else clip_out
             return core.std.FrameEval(clip_out, frameeval_wrapper, prop_src=[clip])
 
-        return cast(F, inner2)
+        return cast(VNodeCallable, inner2)
 
     return inner
 
